// 
// 
#include "ThermostatUDP.h"
#include <WiFiUdp.h>
#include "debugWeb.h"
#include "SystemState.h"
#include "EepromUtil.h"
#include "sensors.h"
#include "ntpTime.h"

unsigned int thermPort = 8888;

const int TPACKET_SIZE = 16;
char packetBuffer[UDP_TX_PACKET_MAX_SIZE / 4];
WiFiUDP udp;								// An UDP instance to let us send and receive packets over UDP

void ThermostatUDP::initializeUDP() {
	udp.begin(thermPort);
}

void ThermostatUDP::writeThermostat() {
	return;

	time_t tnow = ntp.getLocal();
	byte pBuff[TPACKET_SIZE];

	memset(pBuff, 0, sizeof(pBuff));
	pBuff[0] = ss.masterOnOff;
	pBuff[1] = ss.operatingMode;
	pBuff[2] = ss.setRoomTemperature;							//This is the HP set temperature. On the thermostast this is the thermostat temperature (they are different)
	pBuff[3] = ss.currentOperatingState;
	pBuff[4] = (tnow & 0xff000000UL) >> 24;
	pBuff[5] = (tnow & 0x00ff0000UL) >> 16;
	pBuff[6] = (tnow & 0x0000ff00UL) >> 8;
	pBuff[7] = (tnow & 0x000000ffUL);
	pBuff[8] = ss.bThermostatRead;
	pBuff[9] = TEMP_WATER;
	pBuff[10] = TEMP_COMPRESSOR;
	pBuff[11] = TEMP_EVAPORATOR;
	pBuff[12] = TEMP_EXTERNAL;
	pBuff[13] = Defrost.defrostFlag;
	pBuff[14] = 0;												//ID of thermostat
	pBuff[15] = ss.compressorState;

	udp.beginPacket(ss.thermostatIP(), thermPort);
	udp.write(pBuff, sizeof(pBuff));
	udp.endPacket();
	delay(1000);

	int cb = udp.parsePacket();
	if (!cb) {
		ss.bThermostatRead = false;
		DEBUG_PRINTLN(G("no thermostat packet received"));
		DEBUG_PRINTF("%d.%d.%d.%d:%d\n", ss.thermostatIP()[0], ss.thermostatIP()[1], ss.thermostatIP()[2], ss.thermostatIP()[3], thermPort);
	}
	else {
		int newSetting;
		ss.bThermostatRead = true;
		udp.read(pBuff, TPACKET_SIZE);  // read the packet into the buffer
		DEBUG_PRINTLN(G("Read thermostat data"));
		DEBUG_PRINTF("OnOff:%s Temp:%d SetTemp:%d Mode:%s\n", oState[pBuff[0]], pBuff[1], pBuff[2], oMode[pBuff[3]]);
		DEBUG_PRINTF("Start:%d Stop:%d auto:%s\n", pBuff[4], pBuff[5], oState[pBuff[6]]);

		//Only use data if thermostat is set to be used
		if (ss.useThermostat) {
			DEBUG_PRINTLN(G("Thermostat data saved for use"));
			newSetting = pBuff[0];
			if (ss.masterOnOff != newSetting)
				eepromUtil.eeprom_write_byte(E_ONOFF, newSetting);
			newSetting = pBuff[2];
			if (ss.setRoomTemperature != newSetting)
				eepromUtil.eeprom_write_byte(E_ROOMTEMP, newSetting);
			newSetting = pBuff[3];
			if (ss.operatingMode != newSetting)
				eepromUtil.eeprom_write_byte(E_OPERATINGMODE, newSetting);

			newSetting = pBuff[4];
			if (ss.startTime != newSetting)
				eepromUtil.eeprom_write_byte(E_STARTTIME, newSetting);
			newSetting = pBuff[5];
			if (ss.stopTime != newSetting)
				eepromUtil.eeprom_write_byte(E_STOPTIME, newSetting);
			newSetting = pBuff[6];
			if (ss.automatic != newSetting)
				eepromUtil.eeprom_write_byte(E_AUTOMATIC, newSetting);

			//Only switch on/off or change mode from thermostat data if thermostat is active
			ss.masterOnOff = pBuff[0];
			ss.operatingMode = pBuff[3];
			EEPROM.commit();
		}

		//This thermostat data can be used irrespective as it does not affect the machine running
		ss.thermostatTemperature = pBuff[1];
		ss.setRoomTemperature = pBuff[2];
	}

}

void ThermostatUDP::sendUDPJson(DynamicJsonDocument doc) {
	// Send UDP packet
	DEBUG_PRINTLN("sendUDPJson");
	udp.beginPacket(ss.thermostatIP(), thermPort);
	serializeJson(doc, udp);
	udp.println();
	udp.endPacket();
}

// Get JSON Data
void ThermostatUDP::getUDPJson() {
	char sender[16] = { 0 };
	bool status = false;

	StaticJsonDocument<1000> doc;    // Init Json Memory Heap 
	int packetSize = udp.parsePacket();
	if (packetSize)
	{
		DEBUG_PRINTF(G("UDP from port %d size %d\n"), udp.remotePort(), packetSize);
		if ((udp.remotePort() != thermPort)||(packetSize > 50))
		{
			udp.read(packetBuffer, packetSize);
			return;
		}

		IPAddress remote = udp.remoteIP();
		sprintf(sender, "%d.%d.%d.%d", remote[0], remote[1], remote[2], remote[3]);
		DEBUG_PRINTF(G("From %s, port %d\n"), sender, udp.remotePort());
		//DEBUG_PRINTF(G("This IP: %d.%d.%d.%d\n"), WiFi.localIP()[0], WiFi.localIP()[1], WiFi.localIP()[2], WiFi.localIP()[3]);
		//Get UDP Data

		// read the packet into packetBufffer
		udp.read(packetBuffer, packetSize);
		DEBUG_PRINTF("Data Read : %s\n", packetBuffer);

		DeserializationError error = deserializeJson(doc, packetBuffer);

		// Test if parsing succeeds.
		if (error) {
			DEBUG_PRINTF(G("deserializeJson() failed: %s\n"), error.c_str());
			return;
		}
		else {
			DEBUG_PRINTF(G("deserializeJson() succeeded: %s\n"), packetBuffer);
			ss.thermostatTemperature = doc["thermostatTemperature"];
		}
	}
}

void ThermostatUDP::sendStatus() {
	String json = ss.getSystemStatus(false);
	const size_t capacity = 1800;
	DynamicJsonDocument root(capacity);
	deserializeJson(root, json);
	sendUDPJson(root);
}